#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/sysmacros.h>

#include "mctd-log.h"
#include "mctd-mux.h"
#include "mctd-serial.h"
#include "linmuxcfg/linmuxcfg.h"

#define MUX_PORT_NUM 2
#define DRIVER_NODE "/dev/" MUX_CFG_NAME

int mctd_mux_enable()
{
    int iInstance = 0;
    int fd;
    int ret = -1;
    char port_path[PATH_STR_SIZE];

    dev_t devnum = makedev(MUX_FS_MAJOR, 0);

    mknod(DRIVER_NODE, S_IFCHR|0644, devnum);
    fd = open(DRIVER_NODE, 0);
    if (fd == -1) {
        if (_multiplex == MUX_STATUS_ENABLE) {
            perror("open driver node");
            log_warn("open failed for multiplex driver node.");
        }
        goto bail;
    }

    snprintf(port_path, PATH_STR_SIZE, "/dev/%s", SERIAL_TTY);
    if (SetBasePort(fd, iInstance, port_path)) {
        log_warn("Error %d: Failed to set base port %s on driver instance %d\n", ret, port_path, iInstance);
        goto bail;
    }
    if (SetBaudRate(fd, iInstance, 460800)) {
        log_warn("Error %d: Failed to set baud rate %d on driver instance %d\n", ret, SERIAL_BAUDRATE, iInstance);
        goto bail;
    }
    if (SetNumberOfChannels(fd, iInstance, MUX_PORT_NUM)) {
        log_warn("Error %d: Failed to set number of mux channels %d on driver instance %d\n", ret, MUX_PORT_NUM, iInstance);
        goto bail;
    }
    if (SetSwitchoffOn(fd, iInstance, 0)) {
        log_warn("Error %d: Failed to switch switchoff %s on driver instance %d\n", ret, 0 ? "on" : "off", iInstance);
        goto bail;
    }

    ret = 0;

bail:
    close(fd);
    return ret;
}
