#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "mctd-startup-parser.h"

#define COMMENT '#'
#define EQUAL '='

static void erase_comment(char *str)
{
    char *comment;

    comment = strchr(str, COMMENT);
    if (comment)
        *comment = '\0';
}

/**
 * @brief erase space and tab at top and tail.
 * @param str: [in/out] string pointer
 * @return true: has valid string
 *         false: no string
 */
static bool erase_space_and_tab(char **str)
{
    char c;
    size_t len;
    unsigned int i;

    len = strlen(*str);
    while(len > 0) {
        c = (*str)[len -1];
        if (c == ' ' || c == '\t') {
            (*str)[len -1] = '\0';
            len--;
        } else
            break;
    }

    for(i = 0; i < len; i++) {
        if (**str == ' ' || **str == '\t')
            (*str)++;
        else
            break;
    }

    return strlen(*str) > 0;
}

static bool separate_type_and_value(char* str, char **type, char** value)
{
    char *equal;

    equal = strchr(str, EQUAL);
    if (equal) {
        *type = str;
        *equal = '\0';
        *value = &equal[1];
        return true;
    } else {
        *type = NULL;
        *value = NULL;
        return false;
    }
}

static void parse_line(char *str, MCTD_TYPE_AND_VALUE_ARRAY_T *p)
{
    char *type;
    char *value;

    if (!str)
        return;

    erase_comment(str);
    if (strlen(str) <= 0)
        return;

    if (!separate_type_and_value(str, &type, &value))
        return;

    if (!erase_space_and_tab(&type))
        return;

    if (!erase_space_and_tab(&value))
        return;

    if (strlen(value) >= MCTD_STARTUP_MAX_PARAM_LEN)
        return;

    p->tav[p->size].type = strdup(type);
    if (!p->tav[p->size].type) {
        perror("strdup");
        return;
    }

    p->tav[p->size].value = strdup(value);
    if (!p->tav[p->size].value) {
        perror("strdup");
        free(p->tav[p->size].type);
        return;
    }

    sprintf(p->tav[p->size].type, "%s", type);
    sprintf(p->tav[p->size].value, "%s", value);

    p->size++;
}

bool mctd_startup_parse(const char *path, MCTD_TYPE_AND_VALUE_ARRAY_T *p)
{
    FILE *fp;
    char line[256];
    char *cp;

    if (!p)
        return false;

    fp = fopen(path, "r");
    if (!fp) {
        return false;
    }

    memset(p, 0, sizeof(MCTD_TYPE_AND_VALUE_ARRAY_T));

    while (fgets(line, 256, fp)) {
        cp = strchr(line, '\n');
        if (cp)
            *cp = '\0';
        cp = strchr(line, '\r');
        if (cp)
            *cp = '\0';

        parse_line(line, p);
        if (p->size >= MCTD_STARTUP_MAX_PARAMS)
            break;
    }

    fclose(fp);

    return p->size > 0;
}

void mctd_startup_free(MCTD_TYPE_AND_VALUE_ARRAY_T *p)
{
    int i;

    if (!p)
        return;

    for (i = 0; i < p->size; i++) {
        if (p->tav[i].type)
            free(p->tav[i].type);

        if (p->tav[i].value)
            free(p->tav[i].value);
    }
}

char *mctd_startup_get_value(MCTD_TYPE_AND_VALUE_ARRAY_T *p, const char *type)
{
    int i;

    for (i = 0; i < p->size; i++) {
        if (strcmp(p->tav[i].type, type) == 0)
            return p->tav[i].value;
    }
    return NULL;
}
