#include <string.h>
#include <stdlib.h>

#include "mctd.h"
#include "mctd-log.h"
#include "mctd-thread-queue.h"

#define QUEUE_LEN 5

typedef struct {
    int fd;
    unsigned int cmd_id;
    char *param;
    int result;
} QUEUE_DATA;

typedef struct {
    int queue_head;
    int queue_num;
    QUEUE_DATA queue_data[QUEUE_LEN];
} APP_CMD_QUEUE;

static APP_CMD_QUEUE _req_que;
static APP_CMD_QUEUE _res_que;

static int enqueue(APP_CMD_QUEUE *que, int fd, unsigned int cmd_id, char *param, int cmd_result)
{
    int result = MCTD_RESULT_ERR_CMD_BUSY;
    int data_num = (que->queue_head + que->queue_num) % QUEUE_LEN;

    if (que->queue_num < QUEUE_LEN) {
        que->queue_data[data_num].fd = fd;
        que->queue_data[data_num].cmd_id = cmd_id;
        que->queue_data[data_num].result = cmd_result;
        
        if (param != NULL) {
            que->queue_data[data_num].param = malloc(strlen(param) + 1);
            if (!que->queue_data[data_num].param)
                return MCTD_RESULT_ERR_MEMORY;

            strcpy(que->queue_data[data_num].param, param);
            que->queue_data[data_num].param[strlen(param)] = '\0';
        } else
            que->queue_data[data_num].param = NULL;

        que->queue_num ++;
        result = MCTD_RESULT_OK;
    }

    return result;
}

static int dequeue(APP_CMD_QUEUE *que, int *fd, unsigned int *cmd_id, char **param, int *cmd_result)
{
    char *p;
    int result = -1;

    if (que->queue_num > 0) {
        *fd = que->queue_data[que->queue_head].fd;
        *cmd_id = que->queue_data[que->queue_head].cmd_id;
        *cmd_result = que->queue_data[que->queue_head].result;

        p = que->queue_data[que->queue_head].param;
        if (p != NULL) {
            *param = malloc(strlen(p) + 1);
            if (!*param)
                return MCTD_RESULT_ERR_MEMORY;

            memset(*param, 0, strlen(p) + 1);
            strcpy(*param, p);
        }

        que->queue_data[que->queue_head].fd = -1;
        que->queue_data[que->queue_head].cmd_id = MCTD_CMD_NOP;
        free(que->queue_data[que->queue_head].param);
        que->queue_data[que->queue_head].param = NULL;
        que->queue_data[que->queue_head].result = MCTD_RESULT_ERR_CMD_NOT_FOUND;
        que->queue_head = (que->queue_head + 1) % QUEUE_LEN;
        que->queue_num --;
        result = MCTD_RESULT_OK;
    }

    return result;
}

int mctd_thread_queue_req_enqueue(int fd, unsigned int cmd_id, char *param)
{
    return enqueue(&_req_que, fd, cmd_id, param, 0);
}

int mctd_thread_queue_req_dequeue(int *fd, unsigned int *cmd_id, char **param)
{
    int result;

    return dequeue(&_req_que, fd, cmd_id, param, &result);
}

int mctd_thread_queue_res_enqueue(int fd, unsigned int cmd_id, char *param, int result)
{
    return enqueue(&_res_que, fd, cmd_id, param, result);
}

int mctd_thread_queue_res_dequeue(int *fd, unsigned int *cmd_id, char **param, int *result)
{
    return dequeue(&_res_que, fd, cmd_id, param, result);
}

static void queue_init(APP_CMD_QUEUE *que)
{
    que->queue_num = 0;
    que->queue_head = 0;
}

void mctd_thread_queue_init()
{
    queue_init(&_req_que);
    queue_init(&_res_que);
}
